﻿/* 
Microsoft Public License (Ms-PL)

This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.

1. Definitions

The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.

A "contribution" is the original software, or any additions or changes to the software.

A "contributor" is any person that distributes its contribution under this license.

"Licensed patents" are a contributor's patent claims that read directly on its contribution.

2. Grant of Rights

(A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.

(B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.

3. Conditions and Limitations

(A) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.

(B) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.

(C) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.

(D) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.

(E) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement. 
 */
namespace BehaviorLibrary
{
  using System;
  using System.Collections.Generic;
  using System.Windows;
  using System.Windows.Controls;
  using System.Windows.Input;
  using System.Windows.Media;

#if SILVERLIGHT && !WINDOWS_PHONE

  public class DragCompletedGestureEventArgs : EventArgs
  {
    public Orientation Direction { get; set; }
    public bool Handled { get; set; }
  }
  public class GestureListener
  {
    public event EventHandler<DragCompletedGestureEventArgs> DragCompleted;

    internal void RaiseDragCompleted(Orientation orientation)
    {
      if (DragCompleted != null)
      {
        DragCompleted(this, new DragCompletedGestureEventArgs()
        {
          Direction = orientation
        });
      }
    }
  }
  public class GestureService
  {
    private class CurrentTouchData
    {
      Point primary;
      IEnumerable<UIElement> AffectedElements;
      int MaxPointsSeen;

      public void StartTouchOperation(TouchPoint primary)
      {
        this.primary = primary.Position;

        this.AffectedElements = VisualTreeHelper.FindElementsInHostCoordinates(
          this.primary, null);
      }
      public void ProcessNewPoints(TouchPointCollection points)
      {
        this.MaxPointsSeen = Math.Max(this.MaxPointsSeen, points.Count);
      }
      public void EndTouchOperation(TouchPointCollection endPoints)
      {
        if (this.MaxPointsSeen == 1)
        {
          Rect rect = new Rect(this.primary, endPoints[0].Position);

          if (rect.Width > tolerance)
          {
            RaiseEvent(Orientation.Horizontal);
          }
          else if (rect.Height > tolerance)
          {
            RaiseEvent(Orientation.Vertical);
          }
        }
      }
      void RaiseEvent(Orientation orientation)
      {
        foreach (var element in this.AffectedElements)
        {
          GestureListener listener = 
            (GestureListener)element.GetValue(GestureListenerProperty);

          if (listener != null)
          {
            listener.RaiseDragCompleted(orientation);
          }
        }
      }
      const int tolerance = 96 * 2;
    }
    public static readonly DependencyProperty GestureListenerProperty;

    static GestureService()
    {
      GestureListenerProperty = DependencyProperty.RegisterAttached(
        "GestureListener", typeof(GestureListener), typeof(GestureService),
        new PropertyMetadata(null));
    }
    static void EnsureTouchEvents()
    {
      if (!catchingTouchEvents)
      {
        catchingTouchEvents = true;
        Touch.FrameReported += OnTouchFrameReported;
      }
    }
    static void OnTouchFrameReported(object sender, TouchFrameEventArgs e)
    {
      TouchPoint point = e.GetPrimaryTouchPoint(Application.Current.RootVisual);
      TouchPointCollection points = e.GetTouchPoints(Application.Current.RootVisual);

      if (
        (currentTouchData == null) && 
        (point != null) &&
        (points.Count > 0) && 
        (points != null))
      {
        currentTouchData = new CurrentTouchData();
        currentTouchData.StartTouchOperation(point);
      }
      if (
        (points != null) && 
        (points.Count > 0) && 
        (currentTouchData != null) && 
        (points[0].Action == TouchAction.Up))
      {
        currentTouchData.EndTouchOperation(points);
        currentTouchData = null;
      }
      else if ((currentTouchData != null) && (points != null))
      {
        currentTouchData.ProcessNewPoints(points);
      }
    }
    public static void SetGestureListener(DependencyObject theObject,
      GestureListener listener)
    {
      EnsureTouchEvents();
      theObject.SetValue(GestureListenerProperty, listener);
    }
    public static GestureListener GetGestureListener(DependencyObject theObject)
    {
      GestureListener listener = 
        (GestureListener)theObject.GetValue(GestureListenerProperty);

      if (listener == null)
      {
        listener = new GestureListener();
        SetGestureListener(theObject, listener);
      }
      return (listener);
    }
    static CurrentTouchData currentTouchData;
    static bool catchingTouchEvents;    
    const int dragTolerance = 96 * 2;
  }
#endif // SILVERLIGHT && !WINDOWS_PHONE
}